#!/usr/bin/perl -T

################################################################################
# Norton Automatic Updater
# Copyright (C) 2002 Alceu Rodrigues de Freitas Junior 
#(glasswalk3r@yahoo.com.br)
# 
# based on NAI VirusScan DAT file auto updater by Bas Rijniersce (bas@brijn.nu)
#
# Norton Automatic Updater Client version 1
# Copyright (C) 2003 Alex J. Falco (alexjfalcao@hotmail.com)
#
#    These programs are free software; you can redistribute them and/or modify
#    them under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    These programs are distributed in the hope that they will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with these programs; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
################################################################################

#In earlier versions of NAU you might needed to change configuration 
#direct on the script; now, please refer to the file nau.conf for all parameter.

my $conf = shift;

die "You must pass configuration file location as a parameter to nau.pl script.\nFor example: /usr/sbin/nau.pl /etc/nau.conf\n" unless (defined($conf));

###################################################################
# Program Configuration section - DO NOT CHANGE ANYTHING BELLOW !
###################################################################

use strict;
use warnings;
use LWP 5.66;
use URI;
use Digest::MD5;
use Config;

###############################
# system global variables

my $version = '1.7.1';
my @config;
my $local;
my $datlocal;
my $datwebsite;
my $download;
my $file_size;
my $md5sum;
my @content;
my $temp_file = time.'.tmp';
my $path_to_file;


# configuration reading

$| = 1;

open(CONFIG,"<$conf") || die("Cannot open config file $conf: $!\n");

while (<CONFIG>) {
#jumping no interesting lines
    next if $_ =~ /#/;
    chomp;
    next unless ($_ =~ /=/);
# cleaning up values
    $_ =~ s/\s//g;
    my @temp = split(/=/,$_);
    $temp[1] = 'X' unless (defined($temp[1]));
    push (@config,@temp);
}    
close(CONFIG);

my %config = @config;
undef(@config);

foreach ('silent','datdir','siteurl','startup_location','netbios','sharename', 'pattern', 'logging','update') {
    error ("Error: $_ directive has no value at /etc/nau.conf. Aborting...\n") if ($config{$_} eq 'X');
}

# test against tainted directory values

foreach ('datdir','startup_location','logging') {

# test should be different for MS Windows

    if ($Config{osname} =~ /ms\s?win32/i) {

        if  ($config{$_} =~ /(^[\w\_\-\\\:\.]+$)/) {

            $config{$_} = $1;        

        } else {
        
            die "Invalid data in $config{$_}\n";
        
        }

    } else {

        if ($config{$_} =~ /(^[\w\_\-\/\.]+$)/) {
    
            $config{$_} = $1;    
        
        } else {
        
            die "Invalid data in $config{$_}\n" 
    
        }

    }

}

# test against tainted filename

if ($config{update} =~ /(^[\w\-\_\.]+$)/) {

    $config{update} = $1;

} else {

    die "Invalid data in $config{update}\n";

}


# level of verbosity
my $silent = $config{silent};

# $datdir variable tells where the DAT file will be saved in the filesystem
my $datdir = $config{datdir};

# $siteurl specifies where is located the Symantec website to check new versions
my $siteurl = $config{siteurl};

# $startup_location will tell where the script will save the netlogon.bat file
my $startup_location = $config{startup_location};

# the NETBIOS server name to be mapped by MS Windows Stations
my $netbios = $config{netbios};

# the share name that will be mapped by Windows stations
# be sure to create this share in Samba
# using the value of $datdir as the directory
my $sharename = $config{sharename};

# Intelligent Updater file new name
my $update = $config{update};

# Intelligent Updater pattern name
my $pattern = $config{pattern};

# pathname to the logfile
my $logging = $config{logging};

# proxy server, if avaiable
my $proxy = $config{proxy} unless ($config{proxy} eq 'X');

undef %config;

# Get local version
# Its necessary to setup a correct slash to setup the path to definition file

if ($Config{osname} =~ /ms\s?win32/i) {
    $path_to_file = '\\';
} else {
    $path_to_file = '/';
}

# local datinfo file
my $local_datinfo = "$datdir$path_to_file".'definfo.dat';

# local batch file

my $batch = "$startup_location$path_to_file".'netlogon.bat';


###################
#Program execution
###################

# Telling who I am
print "Norton Automatic Updater - $version by <glasswalk3r\@yahoo.com.br>\n\n" if ($silent == 2);


# Do some checking 

print "Checking DAT directory ... " if ($silent >= 1);
if (!-w $datdir) {

    error ("ERROR: DAT dir ($datdir) not found or writable") if ($silent >= 1);

} else { 
 
    print "OK\n" if ($silent >= 1); 
    
}

#Checking website file date last modified date

print "Reading web information about the last update... " if ($silent >= 1);
($datwebsite,$download,$file_size,$md5sum) = check_site($pattern);

print "Reading local information about the last update... " if ($silent >= 1);
$datlocal = checklocal($local_datinfo);

## Let's compare our results...
if ($datwebsite > $datlocal) {

    if ($silent == 2) {
    print "Ok, seems to be I've got work to do.\n";
    print "Getting newer Intelligent Updater file... ";
    }

# getting the file
    get_file($download,'grab_it');

# checking md5sum from the downloaded file
    sleep 2;
    check_md5();

# here we call the batch file creator
    batch($batch,$netbios,$sharename,$update,$datwebsite,$datdir);

# updating datinfo file
    print "Updating the DATINFO in the server with new values... " if ($silent == 2);
    create_datinfo($datwebsite,$local_datinfo);
    print "OK\n";        
    print "I finished it. Everything run ok.\n" if ($silent == 2);
    logging("Downloaded Intelligent Updater file with $file_size, md5sum $md5sum");

} else {
    logging('The DAT file is up to date.');
    print "The DAT file is up to date.\n" if ($silent >= 1);
}

################
# FUNCTION AREA
################


# Function to check the datinfo (date) installed avaiable locally

sub checklocal {
        
    my $local = shift;
    my $date;
    
    unless (-e $local) {
        $date = 1;
        print "There was no DATFILE in $local! I will create a new one if download is successful.\n";

    } else {

        open (DEFFILE,$local) || error ("I can't open the definition file: $!\n");
        while (<DEFFILE>) {
            if (/CurDefs/) {
                # maching only the date numbers
                s/CurDefs=//;
                $date = $_;
                    print "OK\n" if ($silent >= 1);
                print "The local file is from ".localtime($_)."\n" if ($silent == 2);
            } 
        }
        close (DEFFILE);
    }

    chomp($date);
        return($date);
}



# function to create datinfo files
sub create_datinfo {

        my $date = shift;
    my $local = shift;
    
    open (DEF,">$local") || error ("Cannot open the file $local: $!\n");
    print DEF "CurDefs=$date\n";
    print DEF "File was last modified at: ".localtime($date)."\n";
    close (DEF);

}


# function to calcute the DAT file md5sum

sub check_md5 {

# check local file md5sum

    my $md5 = Digest::MD5->new;
    open(FILE,"$datdir$path_to_file$temp_file") || error ("Cant read $datdir$path_to_file$temp_file: $!");
    binmode(FILE);

    print "File was download, now checking md5sum...\n" if ($silent ==2);

    $md5->addfile(*FILE);
    my $md5_result = $md5->hexdigest;
    close(FILE);

    $md5sum = lc($md5sum);
    
    print "Website MD5sum => [$md5sum]\n" if ($silent == 2);
    print "Local MD5sum   => [$md5_result]\n" if ($silent == 2);    

    if ($md5_result eq $md5sum) {
        print "\nVirus definition checksum... OK\n";
        rename "$datdir$path_to_file$temp_file","$datdir$path_to_file$update";

    } else {
        print "\nVirus definition checksum... ERROR\n";
        unlink "$datdir$path_to_file$temp_file";
        error ("The download file did not seems to be valid.\n");    
    }

}

## Function to create LOCK and BATCH file

sub batch {

        my $file      = shift;
    my $netbios   = shift;
    my $sharename = shift;
    my $update    = shift;
    my $date      = shift;
    my $datdir    = shift;
# here comes the lock filename
    my $lock_file = int(rand time / 100).".lock";
    
    print 'Creating the DOS batch file for workstations... ' if ($silent >= 1);
    chdir $datdir;


# and here the batch file

# let's check for a DOS batch file template
    my @template;

    if (-e 'netlogon.template') {
        open (TEMPLATE,'<netlogon.template');
        @template = <TEMPLATE>;
        close(TEMPLATE);
    }

    open (BATCH, ">$file") || error ("cannot open $file: $!");

    if (@template > 0) {

        foreach(@template) { print BATCH;}

    } else {
    
        print "(template file doesn't have a single line in it... but don't tell anyone) " if ($silent == 2);;
    
    }

    undef (@template);
    
        print BATCH "\@echo off\r\n";
        print BATCH "c:\r\n";
        print BATCH "if exist $lock_file goto ok\r\n";
        print BATCH "del *.lock\r\n";
        print BATCH "echo Running Norton Update file...\r\n";
        print BATCH "copy \\\\$netbios\\$sharename\\$update /y\r\n";
        print BATCH "start /w $update /q\r\n";
        print BATCH "del $update\r\n";
        print BATCH "echo Dat file from: $date > $lock_file\r\n";
        print BATCH ":ok\r\n";
        print BATCH "echo Norton AntiVirus is up to date.\r\n";
        print BATCH "exit\r\n";

    close (BATCH);
    
    print "OK\n";
}

# error function
# this function will get all errors messages and put them at a log file
# before ending NAU

sub error {

    my $message = shift;
    chomp($message);

    print "$message\n";

# I won't log anything without reading the configuration file first
    logging($message) if (defined($logging));

    exit(1);
}
    

# function to check website informations:
# - url for download
# - file's size
# - file's md5sum 
# - the Intelligent Updater file date of creation

sub check_site {

    my $word = shift;
    my $test;
    my @list;
    my $download;
    my $date;
    my $file_size;
    my $md5sum;

# getting the html content    
        ($date,@list) = get_url($siteurl);

    for ($test=0;$test<1;$test++) {
        
        my @list = @content;

        foreach (@list) {

# refresh meta tag sucks! this is a workaround
            if (/<meta HTTP-EQUIV="Refresh"/i) {
                s/<meta.*URL=//i;
                s/\/">//;
                ($date,@list) = get_url($_);
                $test--;
                last;
            }

# url matches
            if (/$word/io) {
                $datwebsite = $_;
                $download = clean_tags($_);
                next;    
            }

# file size matches

            if ( (/\d+.\d+\s?MB/i) and (defined($download)) ) {
                $file_size = clean_tags($_);
                last if ( (defined($download)) and (defined($md5sum)) and (defined($datwebsite)) );
                next;
            }

# file md5sum matches

            if ( (/md5/i) and (defined($download)) ) {
                $md5sum = clean_tags($_);
                last if ( (defined($download)) and (defined($md5sum)) and (defined($datwebsite)) );
                next;
            }

        }

    }


# The creation date of Intelligent Updater file            
       $date = get_file($download,'check_it') if (defined($download));
#                $datwebsite =~ s/(<.*?>)+//;
#                    $datwebsite =~ s/(<\/.*?>)+//;
#                $datwebsite =~ s/i32.exe//;
#                $datwebsite =~ s/-[0-9]{3}-//;

# the two guys bellow are vital for operation success

    error ("I was unnable to retrieve the file URL") unless (defined($download));
    error ("I was unnable to define the Intelligent Updater file age") unless (defined($date));

        print "OK\n" if ($silent >= 1);
    print "The website file is from ".localtime($date)."\n";
        return($date,$download,$file_size,$md5sum);    
    
    
}



# this function will clean up html tags for the data wanted
sub clean_tags {
    
    my $line = shift;
    chomp($line);

# if NAU will look for MD5 line
    if (/md5/i) {
        $line =~ s/<\/?.*?>//gi;
        $line =~ s/md5://i;
        $line =~ s/^\s+//;
        $line =~ s/\s+$//;
        $line =~ s/\t+$//;
        $line =~ s/^\t+//;
        $line =~ s/\s+.*//;
            
    } else {

        $line =~ s/<a href=//i;
        $line =~ s/<\a>//i;
        $line =~ s/<\/?.*?>//gi;
        $line =~ s/"//g;
        $line =~ s/^\s+//;
        $line =~ s/\s+$//;
        $line =~ s/\t+$//;
        $line =~ s/^\t+//;
        $line =~ s/>.*//;
    }

    error ("Ooops... theres still garbage... error parsing HTML.") if $line=~ /[<>\!\?\*\&]/;
    error ("Ooops... I think so I cleaned too much... error parsing HTML.") if $line eq '';

    return $line;

}


# this function will read the Symantec website HTML document

sub get_url {

    my $url = shift;
    
    error ("I need a URL as parameter to get HTML content") unless (defined($url));
    
    my $browser = LWP::UserAgent->new;
    $browser->agent('Mozilla 4.0 compatible/MSIE 6.0 Windows 98');

    proxy($browser) if (defined($proxy));

    my $response = $browser->get($url);
    error ("Cant get $url --", $response->status_line) unless ($response->is_success);
    error ("Hey, I was expecting HTML, not ", $response->content_type) unless ($response->content_type eq 'text/html');
    @content = split(/\n/,$response->content);

        return(@content);

}

# this function will check or download the Intelligent Updater file from Symantec website

sub get_file {

    my $url    = shift;
    my $option = shift;
    my $date;

    my $browser = LWP::UserAgent->new;
    $browser->agent('Mozilla 4.0 compatible/MSIE 6.0 Windows 98');
    
    proxy($browser) if (defined($proxy));
    
# taking the absolute URL
    $url = URI->new_abs($url, $siteurl);
    
    error ("I need a URL as parameter to get the file") unless (defined($url));


        if ($option eq 'grab_it') {
         
        $browser->get($url, ':content_file' => "$datdir$path_to_file$temp_file");
        print "OK\n" if ($silent >= 1);
   
    } else {
    
        my $response = $browser->head($url);
        $date = $response->last_modified;
        return($date);
    
    }


}

# logging function, for good or bad news

sub logging {

    my $message = shift;
    chomp($message);

    my $date = localtime;
    
    if (-e "$logging") {
        open(FILE,">>$logging") || die "Error when trying to log information on $logging: $!\n";
    } else {
        open(FILE,">$logging") || die "Error when trying to log information on $logging: $!\n";
    }

    print FILE "[$date] $message\n";
    close(FILE);

}

# create proxy
sub proxy {

    my $browser = shift;
    $browser->proxy(['http','ftp'],"$proxy");

}
